==> resin-4.0.27-src.tar.gz: resin-4.0.27/modules/resin/src/com/caucho/network/listen/ConnectionTask.java <==
/*
 * Copyright (c) 1998-2012 Caucho Technology -- all rights reserved
 *
 * This file is part of Resin(R) Open Source
 *
 * Each copy or derived work must preserve the copyright notice and this
 * notice unmodified.
 *
 * Resin Open Source is software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Resin Open Source is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE, or any warranty
 * of NON-INFRINGEMENT.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Resin Open Source; if not, write to the
 *
 *   Free Software Foundation, Inc.
 *   59 Temple Place, Suite 330
 *   Boston, MA 02111-1307  USA
 *
 * @author Scott Ferguson
 */

package com.caucho.network.listen;

import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.caucho.env.shutdown.ShutdownSystem;

/**
 * Task handling the reading from a connection.
 */
abstract class ConnectionTask implements Runnable {
  private static final Logger log 
    = Logger.getLogger(ConnectionTask.class.getName());
  
  private final TcpSocketLinkListener _listener;
  private final TcpSocketLink _socketLink;
  private SocketLinkThreadLauncher _launcher;
  
  ConnectionTask(TcpSocketLink socketLink)
  {
    _socketLink = socketLink;
    _listener = _socketLink.getListener();
    _launcher = _listener.getLauncher();
  }
  
  protected final TcpSocketLink getSocketLink()
  {
    return _socketLink;
  }
  
  protected final TcpSocketLinkListener getListener()
  {
    return _listener;
  }
  
  protected final SocketLinkThreadLauncher getLauncher()
  {
    return _launcher;
  }
  
  protected boolean isResumeTask()
  {
    return false;
  }
  
  /**
   * Handles the read request for the connection
   */
  abstract RequestState doTask()
    throws IOException;

  @Override
  public final void run()
  {
    SocketLinkThreadLauncher launcher = getLauncher();
    boolean isResume = isResumeTask();
    
    try {
      if (isResume) {
        launcher.onChildIdleEnd();
      }

      runThread();
    } finally {
      if (isResume) {
        launcher.onChildIdleBegin();
      }
    }
  }

  public final void runThread()
  {
    Thread thread = Thread.currentThread();

    RequestState result = RequestState.EXIT;

    _socketLink.startThread(thread);

    try {
      result = doTask();
      
      if (result != null && result.isAcceptAllowed()) {
        if (isResumeTask()) {
          result = _socketLink.handleAcceptTask();
        }
        else {
          result = _socketLink.handleAcceptTaskImpl();
        }
      }
    } catch (OutOfMemoryError e) {
      String msg = "TcpSocketLink OutOfMemory";

      ShutdownSystem.shutdownOutOfMemory(msg); 
    } catch (Throwable e) {
      log.log(Level.WARNING, e.toString(), e);
    } finally {
      _socketLink.finishThread(result);
    }
  }

  @Override
  public String toString()
  {
    return getClass().getSimpleName() + "[" + _socketLink + "]";
  }
}

==> resin-4.0.66-src.tar.gz: resin-4.0.66/modules/resin/src/com/caucho/network/listen/ConnectionTask.java <==
/*
 * Copyright (c) 1998-2018 Caucho Technology -- all rights reserved
 *
 * This file is part of Resin(R) Open Source
 *
 * Each copy or derived work must preserve the copyright notice and this
 * notice unmodified.
 *
 * Resin Open Source is software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Resin Open Source is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE, or any warranty
 * of NON-INFRINGEMENT.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Resin Open Source; if not, write to the
 *
 *   Free Software Foundation, Inc.
 *   59 Temple Place, Suite 330
 *   Boston, MA 02111-1307  USA
 *
 * @author Scott Ferguson
 */

package com.caucho.network.listen;

import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.caucho.env.shutdown.ShutdownSystem;

/**
 * Task handling the reading from a connection.
 */
abstract class ConnectionTask implements Runnable {
  private static final Logger log 
    = Logger.getLogger(ConnectionTask.class.getName());
  
  private final TcpPort _listener;
  private final TcpSocketLink _socketLink;
  private SocketLinkThreadLauncher _launcher;
  
  ConnectionTask(TcpSocketLink socketLink)
  {
    _socketLink = socketLink;
    _listener = _socketLink.getPort();
    _launcher = _listener.getLauncher();
  }
  
  protected final TcpSocketLink getSocketLink()
  {
    return _socketLink;
  }
  
  protected final TcpPort getListener()
  {
    return _listener;
  }
  
  protected final SocketLinkThreadLauncher getLauncher()
  {
    return _launcher;
  }
  
  protected boolean isResumeTask()
  {
    return false;
  }
  
  /**
   * Handles the read request for the connection
   */
  abstract RequestState doTask()
    throws IOException;

  @Override
  public final void run()
  {
    SocketLinkThreadLauncher launcher = getLauncher();
    boolean isResume = isResumeTask();
    
    try {
      if (isResume) {
        launcher.onChildIdleEnd();
      }

      runThread();
    } finally {
      if (isResume) {
        launcher.onChildIdleBegin();
      }
    }
  }

  public final void runThread()
  {
    Thread thread = Thread.currentThread();

    RequestState result = RequestState.EXIT;

    _socketLink.startThread(thread);

    try {
      result = doTask();
      
      if (result != null && result.isAcceptAllowed()) {
        if (isResumeTask()) {
          result = _socketLink.handleAcceptTask();
        }
        else {
          result = _socketLink.handleAcceptTaskImpl();
        }
      }
    } catch (OutOfMemoryError e) {
      String msg = "TcpSocketLink OutOfMemory";

      ShutdownSystem.shutdownOutOfMemory(msg); 
    } catch (Throwable e) {
      log.log(Level.WARNING, e.toString(), e);
    } finally {
      _socketLink.finishThread(result);
    }
  }

  @Override
  public String toString()
  {
    return getClass().getSimpleName() + "[" + _socketLink + "]";
  }
}
